@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2001 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2001-2002 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
  *
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -29,60 +30,77 @@
 #include "gmime-filter-basic.h"
 #include "gmime-utils.h"
 
-static void filter_destroy (GMimeFilter *filter);
+static void g_mime_filter_basic_class_init (GMimeFilterBasicClass *klass);
+static void g_mime_filter_basic_init (GMimeFilterBasic *filter, GMimeFilterBasicClass *klass);
+static void g_mime_filter_basic_finalize (GObject *object);
+
 static GMimeFilter *filter_copy (GMimeFilter *filter);
-static void filter_filter (GMimeFilter *filter, char *in, size_t len, 
-			   size_t prespace, char **out, 
-			   size_t *outlen, size_t *outprespace);
-static void filter_complete (GMimeFilter *filter, char *in, size_t len, 
-			     size_t prespace, char **out, 
-			     size_t *outlen, size_t *outprespace);
+static void filter_filter (GMimeFilter *filter, char *in, size_t len, size_t prespace,
+			   char **out, size_t *outlen, size_t *outprespace);
+static void filter_complete (GMimeFilter *filter, char *in, size_t len, size_t prespace,
+			     char **out, size_t *outlen, size_t *outprespace);
 static void filter_reset (GMimeFilter *filter);
 
-static GMimeFilter filter_template = {
-	NULL, NULL, NULL, NULL,
-	0, 0, NULL, 0, 0,
-	filter_destroy,
-	filter_copy,
-	filter_filter,
-	filter_complete,
-	filter_reset,
-};
+
+static GMimeFilterClass *parent_class = NULL;
 
 
-/**
- * g_mime_filter_basic_new_type:
- * @type: filter type
- *
- * Creates a new filter of type @type.
- *
- * Returns a new basic filter of type @type.
- **/
-GMimeFilter *
-g_mime_filter_basic_new_type (GMimeFilterBasicType type)
+GType
+g_mime_filter_basic_get_type (void)
 {
-	GMimeFilterBasic *new;
+	static GType type = 0;
 	
-	new = g_new (GMimeFilterBasic, 1);
+	if (!type) {
+		static const GTypeInfo info = {
+			sizeof (GMimeFilterBasicClass),
+			NULL, /* base_class_init */
+			NULL, /* base_class_finalize */
+			(GClassInitFunc) g_mime_filter_basic_class_init,
+			NULL, /* class_finalize */
+			NULL, /* class_data */
+			sizeof (GMimeFilterBasic),
+			0,    /* n_preallocs */
+			(GInstanceInitFunc) g_mime_filter_basic_init,
+		};
+		
+		type = g_type_register_static (GMIME_TYPE_FILTER, "GMimeFilterBasic", &info, 0);
+	}
 	
-	new->type = type;
-	new->state = 0;
-	new->save = 0;
-	
-	g_mime_filter_construct (GMIME_FILTER (new), &filter_template);
-	
-	filter_reset (GMIME_FILTER (new));
-	
-	return GMIME_FILTER (new);
+	return type;
 }
 
 
 static void
-filter_destroy (GMimeFilter *filter)
+g_mime_filter_basic_class_init (GMimeFilterBasicClass *klass)
 {
-	g_free (filter);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GMimeFilterClass *filter_class = GMIME_FILTER_CLASS (klass);
+	
+	parent_class = g_type_class_ref (GMIME_TYPE_FILTER);
+	
+	object_class->finalize = g_mime_filter_basic_finalize;
+	
+	filter_class->copy = filter_copy;
+	filter_class->filter = filter_filter;
+	filter_class->complete = filter_complete;
+	filter_class->reset = filter_reset;
 }
 
+static void
+g_mime_filter_basic_init (GMimeFilterBasic *filter, GMimeFilterBasicClass *klass)
+{
+	filter->type = 0;
+	filter->state = 0;
+	filter->save = 0;
+}
+
+static void
+g_mime_filter_basic_finalize (GObject *object)
+{
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
 static GMimeFilter *
 filter_copy (GMimeFilter *filter)
 {
@@ -254,3 +272,24 @@ filter_reset (GMimeFilter *filter)
 	}
 	basic->save = 0;
 }
+
+
+/**
+ * g_mime_filter_basic_new_type:
+ * @type: filter type
+ *
+ * Creates a new filter of type @type.
+ *
+ * Returns a new basic filter of type @type.
+ **/
+GMimeFilter *
+g_mime_filter_basic_new_type (GMimeFilterBasicType type)
+{
+	GMimeFilterBasic *new;
+	
+	new = g_object_new (GMIME_TYPE_FILTER_BASIC, NULL, NULL);
+	
+	filter_reset ((GMimeFilter *) new);
+	
+	return (GMimeFilter *) new;
+}
